    member

    !
    ! changes and additions are tagged with LGW
    !

    include('trackbar.inc')

HandlerQueue    queue       ! LGW, qHwnd was removed from queue
qControlHwnd      HWND
qControl          &cControl
qControlLong      LONG      ! LGW
                end

ClientQueue     QUEUE       ! LGW, window client subclass handler queue
c_hWnd            LONG
c_Long            LONG
                END

    map
      module('Windows')
        CreateWindowEx(DWORD,*LPCSTR,*LPCSTR, DWORD, SIGNED, SIGNED, SIGNED, SIGNED, HWND, HMENU, HINSTANCE, LPVOID), HWND, PASCAL, RAW, name('CreateWindowExA')
        SendMessage(HWND,UNSIGNED,WPARAM,LPARAM),LRESULT,PASCAL,NAME('SendMessageA')
        CallWindowProc(LONG,HWND,unsigned,WPARAM,LPARAM),LONG,RAW,PASCAL,NAME('CallWindowProcA')
        SetWindowLong(HWND,signed,LONG),LONG,RAW,PASCAL,NAME('SetWindowLongA')
        InitCommonControls(), PASCAL, NAME('InitCommonControls')
        DefWindowProc(HWND,UNSIGNED,WPARAM,LPARAM),LRESULT,PASCAL,NAME('DefWindowProcA')
        ! LGW
        ShowWindow(UNSIGNED,SIGNED),BOOL,PASCAL,NAME('ShowWindow'),PROC
        SetFocus(UNSIGNED),UNSIGNED,PASCAL,NAME('SetFocus'),PROC
        SetWindowPos(UNSIGNED,UNSIGNED,SIGNED,SIGNED,SIGNED,SIGNED,UNSIGNED),SIGNED,PASCAL,NAME('SetWindowPos'),PROC
        EnableWindow(UNSIGNED,BOOL),BOOL,PASCAL,NAME('EnableWindow'),PROC
        PostMessage(UNSIGNED, UNSIGNED, UNSIGNED, LONG),BOOL,PASCAL,NAME('PostMessageA'),PROC
        !
      end
      TBWinProc(UNSIGNED MyHwnd, UNSIGNED usMsg, UNSIGNED WParam, LONG lParam),LONG,PASCAL
      TBClientProc(UNSIGNED tbc_hWnd, UNSIGNED tbc_Msg, UNSIGNED tbc_wParam, LONG tbc_lParam),LONG,PASCAL   ! LGW
    end


cControl.WndProc    function(unsigned MyhWnd, unsigned usMsg, unsigned WParam, long lParam)

    code
    return(true)

cControl.CheckHwnd  function()

    code
    if ~SELF.m_Hwnd
      return(false)
    end
    return(true)

cControl.Notification   function()

    code
    return(SELF.m_Notification)


! --------------------------------------------------------------
cTrackBar.CreateTB      function(long pRegion, unsigned pParentHwnd)
! --------------------------------------------------------------
! This method creates the trackbar control and return true if the control was
! created successfully.
! pRegion is a FEQ of a region control to get a screen location on the window
! pParent is the Parent Handle of the control. The control will send notification
! message to that hanlde.

    code
    ! First, make sure to make Windows happy and call the common controls init code.
    InitCommonControls()

    ! Is there a style defined? if not, give a default.
    if ~SELF.m_Style
      SELF.m_Style = bor(WS_CHILD, bor(WS_TABSTOP, bor(WS_VISIBLE, TBS_AUTOTICKS)))
    end

    ! Check if the parent window has prop:pixels set to true
    if ~0{prop:pixels}
      0{prop:pixels} = true
    end

    ! Create the control
    SELF.m_Hwnd = CreateWindowEx(SELF.m_StyleEx, |
                               SELF.m_ClassName, |
                               SELF.m_cWindowName, |
                               SELF.m_Style , |
                               pRegion{prop:XPos} + 2, |
                               pRegion{prop:YPos} + 2, |
                               pRegion{prop:Width} - 2, |
                               pRegion{prop:Height} - 2, |
                               pParentHwnd, |
                               SELF.m_ControlID, 0, 0)


    ! Check if the HWND is valid.
    if ~SELF.CheckHwnd()
      return(false)
    end

    !
    ! LGW, subclass the window client area to handle scroll messages
    !      to be posted to the trackbar control subclass
    !
    ClientQueue.c_hWnd = pParentHwnd
    GET(ClientQueue,ClientQueue.c_hWnd)
    IF ERRORCODE()
      ClientQueue.c_hWnd    = pParentHwnd
      ClientQueue.c_Long    = 0{PROP:ClientWndProc}
      0{PROP:ClientWndProc} = ADDRESS(TBClientProc)
      IF NOT ClientQueue.c_Long
        RETURN( FALSE )
      END
      ADD(ClientQueue,ClientQueue.c_hWnd)
    END

    !
    ! LGW, subclass the control to receive messages from window subclass via PostMessage
    !
    HandlerQueue.qControlLong = SetWindowLong( SELF.m_hwnd, GWL_WndProc, ADDRESS(TBWinProc) )

    ! Grab the current thread number
    SELF.m_ThreadID = thread()

    ! Now, populate the handler queue with appropriate value.
    HandlerQueue.qControl &= SELF
    HandlerQueue.qControlHwnd = SELF.m_hwnd

    add(HandlerQueue, HandlerQueue.qControlHwnd)
    if errorcode()
      return(false)
    end


    !
    ! LGW, Automatically re-order tab stops and HIDE THE SLIDER
    !
    ! This works although if the slider is the first control on a
    ! window or tab, Clarion has no knowledge of its existance and
    ! cannot automatically select it since the slider is not a
    ! Clarion control. In this case use the Select method at runtime
    ! after the window is opened or the tab is selected.
    !
    ! If the TrackBar is NOT the first control on the window that
    ! can receive focus, the changes below place it in the correct
    ! tab order.
    !
    SELF.Follows( pRegion-1 )
    SELF.Hide


    ! We are doing good so far! Now kill the region control
    destroy(pRegion)
    return(true)

! ------------------------------------
cTrackBar.GetLowerLimit     function()
! ------------------------------------
! Return the Lower limit of the control. This is
! the lower value the Slider can be positioned over.

    code
    return(SELF.m_Lower)

! ------------------------------------
cTrackBar.GetUpperLimit     function()
! ------------------------------------
! Return the UPPER limit of the control. This is
! the upper value the Slider can be positioned over.

    code
    return(SELF.m_Upper)

! --------------------------------
cTrackBar.GetStart      function()
! --------------------------------
! Return the Start value of the range
! defined in the Channel Rectangle of the control.

    code
    if ~SELF.CheckHWnd()
      return(0)
    end
    return(SendMessage(SELF.m_Hwnd, TBM_GETSELSTART, 0, 0))

! ------------------------------
cTrackBar.GetEnd      function()
! ------------------------------
! Return the End value of the range
! defined in the Channel Rectangle of the control.

    code
    if ~SELF.CheckHWnd()
      return(0)
    end
    return(SendMessage(SELF.m_Hwnd, TBM_GETSELEND, 0, 0))

! ------------------------------
cTrackBar.GetPos      function()
! ------------------------------

    code
    return(SendMessage(SELF.m_Hwnd, TBM_GETPOS, 0, 0))

! ---------------------------------------
cTrackBar.SetPos      function(long pPos)
! ---------------------------------------

    code
    if SendMessage(SELF.m_Hwnd, TBM_SETPOS, true, pPos).  ! The true tells the control to repaint itself.
    return(true)

! ----------------------------------------------------
cTrackBar.SetLimit  function(long pLower, long pUpper)
! ----------------------------------------------------
! Set the limit of the trackbar control. The limit is the
! range of values the control can represent.

gLParam     group
lWord         short
hWord         short
            end

lParam      long, over(gLParam)

    code
    assert(SELF.m_hwnd)
    if ~SELF.CheckHWnd()
      return(false)
    end

    gLParam.lWord = pLower
    gLParam.hWord = pUpper
    SELF.m_Lower = pLower
    SELF.m_Upper = pUpper
  
    if SendMessage(SELF.m_Hwnd, TBM_SETRANGE, true, lParam). ! The true will tell the control to repaint itself.

    return(true)


! ----------------------------------------------
cTrackBar.SetPageSize       function(long pPage)
! ----------------------------------------------
! SetPageSize will set the number of position the Slider will
! move when the user click on the slider bar

    code
    assert(SELF.m_HWnd)
    if ~SELF.CheckHWnd()
      return(false)
    end

    if SendMessage(SELF.m_Hwnd, TBM_SETPAGESIZE, 0, pPage).
    return(true)


! ------------------------------------------------------
cTrackBar.SetRange      function(long pStart, long pEnd)
! ------------------------------------------------------
! SetRange will define the range displayed in the Channel Rectangle
! of the control. The control must have the TBS_ENABLESELRANGE bit style set
! to properly use this method.

gLParam     group
lWord         short
hWord         short
            end

lParam      long, over(gLParam)

    code
    assert(SELF.m_Hwnd)
    if ~SELF.CheckHwnd()
      return(false)
    end

    gLParam.lWord = pStart
    gLParam.hWord = pEnd
    if SendMessage(SELF.m_Hwnd, TBM_SETSEL, true, lParam). ! The true will tell the control to repaint itself.
    return(true)

! ------------------------------------------
cTrackBar.SetTicsFreq   function(long pFreq)
! ------------------------------------------
! SetTicsFreq will tell the control to place tics marks
! every pFreq along the control

    code
    assert(SELF.m_Hwnd)
    if ~SELF.CheckHwnd()
      return(false)
    end
    if SendMessage(SELF.m_HWnd, TBM_SETTICFREQ, pFreq, 0).
    return(true)

! -----------------------------------------
cTrackBar.SetTicAt      function(long pTic)
! -----------------------------------------
! SetTicAt will place a tic mark at the position
! specified by the pTic parameter.

    code
    if SendMessage(SELF.m_Hwnd, TBM_SETTIC, 0, pTic) = -1
      return(false)
    end
    return(true)


cTrackBar.TakeNotification      function()

    code
    return(true)

! -----------------------------------------------------------------------------------------
cTrackBar.WndProc       function(unsigned MyhWnd, unsigned usMsg, unsigned WParam, long lParam)
! -----------------------------------------------------------------------------------------

gWParam     group,over(wParam)
lWord         short
hWord         short
            end

        code

        !
        ! LGW, reload HandlerQueue
        !
        HandlerQueue.qControlHwnd = MyHwnd
        GET(HandlerQueue, HandlerQueue.qControlHwnd)
        IF ERRORCODE() THEN RETURN(1).

        case usMsg
        of WM_HSCROLL
        orof WM_VSCROLL
          SELF.m_Notification = gWParam.lWord
          ! By default, we want to get the slider position
          case gWParam.lWord
          of TB_THUMBPOSITION
          orof TB_THUMBTRACK
            SELF.m_SliderPos = gWParam.hWord
          else
            SELF.m_SliderPos = SendMessage(lParam, TBM_GETPOS, 0, 0)
          end
          ! And whatever we want to do.
          if ~SELF.TakeNotification()
            return(false)
          end
          post(0, , SELF.m_ThreadID) ! This will wake up the accept loop of the parent window
        end
        RETURN( CallWindowProc( HandlerQueue.qControlLong, MyHwnd, usMsg, WParam, lParam ) )


!
! LGW, created ONCE for each TrackBar on a given window
!
! TrackBar control is subclassed to receive scroll event messages from
! parent window client area... subclassed below.
!
TBWinProc PROCEDURE(UNSIGNED MyHWnd, UNSIGNED usMsg, UNSIGNED wParam, LONG lParam)

  CODE
  HandlerQueue.qControlHwnd = MyhWnd
  GET(HandlerQueue, HandlerQueue.qControlHwnd)
  IF ERRORCODE() THEN RETURN(1).                            ! should never happen, but just in case...
  IF usMsg = WM_HSCROLL OR usMsg = WM_VSCROLL
    RETURN( HandlerQueue.qControl.WndProc( MyhWnd, usMsg, wParam, MyhWnd ) )
  END
  RETURN( CallWindowProc( HandlerQueue.qControlLong, MyHwnd, usMsg, WParam, lParam ) )


!
! LGW, created ONCE for each window
!
! Window client is subclassed to receive scroll event messages and
! post them to the control.
!
TBClientProc PROCEDURE(UNSIGNED tbc_hWnd, UNSIGNED tbc_Msg, UNSIGNED tbc_wParam, LONG tbc_lParam)

  CODE
  ClientQueue.c_hWnd = tbc_hWnd
  GET(ClientQueue,ClientQueue.c_hWnd)
  IF ERRORCODE() THEN RETURN(1).                            ! should never happen, but just in case...
  IF tbc_lParam AND (tbc_Msg = WM_HSCROLL OR tbc_Msg = WM_VSCROLL)
    HandlerQueue.qControlHwnd = tbc_lParam
    GET(HandlerQueue, HandlerQueue.qControlHwnd)
    IF NOT ERRORCODE()                                      ! this is a trackbar, post message
      PostMessage(tbc_lParam, tbc_Msg, tbc_wParam, 0)
    END
  END
  RETURN( CallWindowProc( ClientQueue.c_Long, tbc_hWnd, tbc_Msg, tbc_wParam, tbc_lParam ) )


cTrackBar.Disable PROCEDURE
  !
  ! LGW, Disable the control
  !
  CODE
  IF SELF.CheckHWnd()
    EnableWindow(SELF.m_Hwnd,0)
  END
  RETURN


cTrackBar.Enable PROCEDURE
  !
  ! LGW, Enable the control
  !
  CODE
  IF SELF.CheckHWnd()
    EnableWindow(SELF.m_Hwnd,1)
  END
  RETURN


cTrackBar.Hide PROCEDURE
  !
  ! LGW, Hide the control
  !
  ! This is required when using sheets and tabs since Clarion cannot
  ! handle hiding the control with the runtime.
  !
  ! Use EVENT:NewSelection for the sheet, such as:
  !
  ! OF EVENT:NewSelection
  !   CASE FIELD()
  !     OF ?Sheet1
  !       EXECUTE CHOICE(?Sheet)
  !         BEGIN
  !           ????.Unhide
  !           ????.Hide
  !           ????.Hide
  !         END
  !         BEGIN
  !           ????.Hide
  !           ????.Unhide
  !           ????.Hide
  !         END
  !         BEGIN
  !           ????.Hide
  !           ????.Hide
  !           ????.Unhide
  !         END
  !
  ! NOTE: Remember that the changes to the class automatically
  !       hide the control once it is created ! ! !
  !
  CODE
  IF SELF.CheckHWnd()
    ShowWindow(SELF.m_Hwnd,0)
  END
  RETURN


cTrackBar.Unhide PROCEDURE
  !
  ! LGW, Unhide the control
  !
  CODE
  IF SELF.CheckHWnd()
    ShowWindow(SELF.m_Hwnd,1)
  END
  RETURN


cTrackBar.Select PROCEDURE
  !
  ! LGW, Select the control
  !
  CODE
  IF SELF.CheckHWnd()
    SetFocus(SELF.m_Hwnd)
  END
  RETURN


cTrackBar.Follows PROCEDURE(LONG pFEQ)
  !
  ! LGW, Reset control tab order
  !
  ! SWP_NOSIZE          EQUATE(1)         1 +
  ! SWP_NOMOVE          EQUATE(2)         2 +
  ! SWP_NOREDRAW        EQUATE(8)         8 +
  ! SWP_NOACTIVATE      EQUATE(010h)     16 +
  ! SWP_NOOWNERZORDER   EQUATE(0200h)   512 = 539
  !
  CODE
  IF SELF.CheckHWnd()
    IF pFEQ{PROP:Handle}
      RETURN( SetWindowPos( SELF.m_Hwnd, pFEQ{PROP:Handle}, 0, 0, 0, 0, 539 ) )
    END
  END
  RETURN(0)

